#-----------------------------------------------------------------------------
# SteadiCam
# Aug 15, 2005
# Copyright (c) 2005 by David G. Drumright (ockham)
#-----------------------------------------------------------------------------
#-----------------------------------------------------------------------------

from Tkinter import *
import poser
scene = poser.Scene()
LastFrame=scene.NumFrames()-1

# Requires you to point, NOT PARENT, the camera at a body part and adjust before use.
# We just move the camera parallel with the Body, smoothing as we go.

#-----------------------------------------------------------------------------

SelFig=scene.CurrentFigure()
Body=SelFig.Actor("BODY")
SelCam=scene.CurrentCamera()

camParmX=SelCam.ParameterByCode(poser.kParmCodeXTRAN)
camParmY=SelCam.ParameterByCode(poser.kParmCodeYTRAN)
camParmZ=SelCam.ParameterByCode(poser.kParmCodeZTRAN)
bodParmX=Body.ParameterByCode(poser.kParmCodeXTRAN)
bodParmY=Body.ParameterByCode(poser.kParmCodeYTRAN)
bodParmZ=Body.ParameterByCode(poser.kParmCodeZTRAN)

#-----------------------------------------------------------------------------
# Next section is TK
#------------------------------------------------------------

class App:
    def __init__(self, master, textMessage):
        self.master = master
        master.title("SteadiCam")

        self.StatusEntry = Entry(self.master,width=60)
        self.StatusEntry.grid(row=0,column=0,columnspan=3)
        self.ShowStatus("Set chosen smoothing value, hit Go.")

        self.SmoothSlider = Scale(self.master, from_=0, to=80,orient=HORIZONTAL,resolution=1.0)
        self.SmoothSlider.grid(row=2, column=0,columnspan=3,padx=2,pady=2)
        self.SmoothSlider.set(20)

        # Go/nogo buttons.

        self.buttonGo = Button(self.master, text="Go", command=self.HandleGo)
        self.buttonGo.grid(row=3, column=0,padx=2,pady=2)

        self.buttonGo = Button(self.master, text="Go flat", command=self.HandleGoFlat)
        self.buttonGo.grid(row=3, column=1,padx=2,pady=2)

        self.buttonCancel = Button(self.master, text="Close", command=self.HandleCancel)
        self.buttonCancel.grid(row=3, column=2,padx=2,pady=2)

        self.master.protocol('WM_DELETE_WINDOW', self.HandleCancel)

    # - - - - - - - - - - - - - - - - - -

    def ShowStatus(self,S): # Just saving repetition
        self.StatusEntry.delete(0,END)
        self.StatusEntry.insert(0,S)
        self.StatusEntry.update_idletasks()

    # - - - - - - - - - - - - - - - - - -

    def HandleGo(self):
        self.Action(0)

    # - - - - - - - - - - - - - - - - - -

    def HandleGoFlat(self):
        self.Action(1)

    # - - - - - - - - - - - - - - - - - -

    def Action(self,Flat):
        # Pick up the slider
        self.ShowStatus("Running....")
        Percent=self.SmoothSlider.get() * 0.01
        Span = Percent * float(LastFrame)
        HalfSpan = int(Span)/2

        # Arrays for raw and av positions of camera
        self.OrigX={}
        self.OrigY={}
        self.OrigZ={}

        self.AvX={}
        self.AvY={}
        self.AvZ={}

        # Get the starting arrangement
        camOldX=camParmX.Value()
        camOldY=camParmY.Value()
        camOldZ=camParmZ.Value()
        bodOldX=bodParmX.Value()
        bodOldY=bodParmY.Value()
        bodOldZ=bodParmZ.Value()
        OrigDistX=camOldX-bodOldX
        OrigDistY=camOldY-bodOldY
        OrigDistZ=camOldZ-bodOldZ

        for F in range(LastFrame): # First get the body pos
            scene.SetFrame(F)
            self.ShowStatus("Working at frame %d" % F)
            scene.DrawAll()
            # Get the new pos of body:
            self.OrigX[F]=bodParmX.Value()
            self.OrigY[F]=bodParmY.Value()
            self.OrigZ[F]=bodParmZ.Value()

        # Now smooth these positions before using them
        for F in range(LastFrame):
            ax=0.0
            ay=0.0
            az=0.0
            ActualCount=0.0
            for subF in range( (F-HalfSpan),(F+HalfSpan+1) ):
                if subF < 0: continue
                elif subF >= LastFrame: break
                ax=ax+self.OrigX[subF]
                ay=ay+self.OrigY[subF]
                az=az+self.OrigZ[subF]
                ActualCount=ActualCount+1.0
            if ActualCount==0.0:
               self.AvX[F]=self.OrigX[F]
               self.AvY[F]=self.OrigY[F]
               self.AvZ[F]=self.OrigZ[F]
            else:
               self.AvX[F]=ax/ActualCount
               self.AvY[F]=ay/ActualCount
               self.AvZ[F]=az/ActualCount

        # Now move the cam's values so that it stays ahead of the average by
        # the original distance.

        for F in range(LastFrame):
            scene.SetFrame(F)
            camParmX.SetValue(self.AvX[F]+OrigDistX)
            if not Flat:
                camParmY.SetValue(self.AvY[F]+OrigDistY)
            camParmZ.SetValue(self.AvZ[F]+OrigDistZ)

        scene.SetFrame(0) #Back
        self.ShowStatus("Done.")


    # - - - - - - - - - - - - - - - - - -

    def Update(self):
        if scene: scene.ProcessSomeEvents()
        root.lift()
        root.after(100, self.Update)

    # - - - - - - - - - - - - - - - - - -

    def HandleCancel(self):
        self.master.destroy()

#------------------------------------------------------------
# Activate the loop
#------------------------------------------------------------

root = Tk()
app = App(root, '')
app.Update()
root.mainloop()

#------------------------------------------------------------
# End TK loop.
#-----------------------------------------------------------------------------
